/*
   We have added a bit from our "Very Basic Autonomous" sketch to see if we can get a bit smoother operation. 
   We started with a little chunk of code in the readSensor() routine that will give us a left and right average. The variable rangeL is the average of the left and center sensors.
   The variable rangeR is the average of the center and right sensors. From there, our code is still fairly simple:
   
   Read the sensor
   if the sensor average(for the whole sensor) is closer than danger, then we need to turn
   if L is closer than R, then turn right for 1 second
   if R is closer than L then turn left for 1 second
   if none of the above is true, just go forward instead
   do it all again (loop back to the start)
   
   Your "danger" level should be pretty close to what it was in the "Very Basic Autonomous" sketch, you should not have to change it, but it wouldn't hurt to play around with it
   and observe the different actions of the bot.
   
   You can change your speeds and can also change the variable "turnTime". turnTime is how long the robot will turn away from an obstacle. One second (1000 mS) seemed to work for me
   you should play with different pauses and see what you get.
   
   You can also change the "slides" (one wheel forward/ one wheel off) to "spins" (one wheel forward/ one wheel reverse) if you would like as well.

   
   ******REMEMBER*****  When you first turn your robot on, it is going to read its sensors to determine the offsets for each sensor. BE SURE your robot is sitting on the place where it will
   be driving around and that there is nothing in front or to the sides of your bot at start up. **If you notice, a 3-second delay has been added to the setup routine. This will give you time
   to physically set the robot down after flipping the switch on.
   
   
   ********    Use <Arduino Pro or Pro Mini (3.3V 8MHz) w/ ATmega168> under the "Tools" menu  ********** 
*/

#include <microM.h> //this tells the arduino to use the "microM" library
#define obstaclePinL 3  //remember these are analog pins (A3, A4, A5) they are right next to the USB on the MicroM board
#define obstaclePinC 4
#define obstaclePinR 5
#define obstacleOnOffPin 13 //this is a digital pin (13) that we will use to turn the LED's on and off on the obstacle sensor
#define off 0 //we will use this as speed=0 for our motors  ("off" is just easier to remember and looks better when reading the code)
#define DEBUG 0  // set to 1 to print to serial monitor, 0 to disable

int obstacleReadings[3]; // this is our array. Think of it as 3 mailboxes (variables) but they are all in a row, share the same name, and numbered 0,1,2 etc.
int obstacleReadingOffsets[3]; // these will be our "fudge" numbers to deal with any inconsistancies in our readings
int sensorAverage=0; //average of 3 sensors
int rangeL; //later we will average the left and center sensors to get an overall left-side measurement --We will store that number here
int rangeR; //later we will average the center and right sensors to get an overall right-side measurement --We will store that number here
int i;

int forwardSpeedL=175;  // max 255, mid 175, min 75  - Left motor is slower on this bot. Your bot may vary.
int forwardSpeedR=120;  // max 187, mid 120, min 60  - Right motor can always catch left, so the drivestraight function sets right to match left.
int reverseSpeedL=-160;
int reverseSpeedR=-150;

int danger=70; //When we just want to start turning
int turnTime=1000;  //how long we will turn when we see an obstacle

// Encoder tracking stuff
volatile long countL=0;  // for encoders
volatile long countR=0;  // for encoders
long countRef=8;     // reference number of encoder ticks to count
long old_countL=0;   // stores previous left encoder count
long old_countR=0;   // stores previous right encoder count
int speedRef=180;    // reference speed to match. Not currently used, but if we want to match a set speed, set speedL = speedRef.
int errorL_p=0;      // Left encoder error - P: Proportional
int old_errorL=0;    // Used to store previous error
int errorL_d=0;      // Left encoder error - D: Derivative
int errorL_i=0;      // Left encoder error - I: Integral
int errorR_p=0;      // Right encoder error - P: Proportional
int old_errorR=0;    // Used to store previous error
int errorR_d=0;      // Right encoder error - D: Derivative
int errorR_i=0;      // Right encoder error - I: Integral

void setup()
{
  microM.Setup(); //fire up the library
  pinMode(obstacleOnOffPin,OUTPUT); //this is the output pin we will use to turn the obstacle sensor on and off
  Serial.begin(19200); //Hey arduino, we are going to talk to the computer (the serial monitor) at 19200 baud (baud is how fast we are going to "talk")
  delay(3000);  //gives you 3 seconds to put the robot down on the table
  initSensorReadings(); //take first reading of our sensor ***Remember, the bot needs to be on the table and looking at infinity

// Stuff for encoders  
  attachInterrupt(0,intCountL,CHANGE);
  attachInterrupt(1,intCountR,CHANGE);
  digitalWrite(2,1);                               // enable pullup resistor on D2
  digitalWrite(3,1);                           // this must be called to initialize timer 2
}

void loop()
{


  readSensor();
  //printSensorReadings();
  if(sensorAverage>danger) //if either L or R is too close, we gotta turn one way or the other
  {
    if(rangeL > rangeR) //if the obstacle is closer on the left, then we should turn right
    {
      microM.Motors(forwardSpeedL,reverseSpeedR,0,0); //right slide
      delay(turnTime);
    }else{               //else, it is closer on the right and we need to turn left
      microM.Motors(reverseSpeedL,forwardSpeedR,0,0); //left slide
      delay(turnTime);
    }
  }else{
//    microM.Motors(forwardSpeedL,forwardSpeedR,0,0); //go forward, pretty simple
      drivestraight();
  }


}

void drivestraight(){
    microM.Motors(forwardSpeedL,forwardSpeedR,0,0); //go forward, using speeds set by encoder feedback
    float SpeedLadjust = forwardSpeedL;    // Need float variables to apply non-integer P, D, and I gains
    float SpeedRadjust = forwardSpeedR;
    float Kp=2.0;  // Proportional gain
    float Kd=2.0;  // Derivative gain
    float Ki=0.2;  // Integral gain

    old_countL=countL;    // Set the old count equal to the current count.
    old_countR=countR;    // When we enter the if loop below, we'll need the old count to be the most recent count.
    errorL_i = 0;         // Reset the integral errors, or they will just keep adding up and up!
    errorR_i = 0;

    if(countL>=countRef){
      errorL_p = countL - countRef;                // Difference between the current count and the reference count: Proportional error
      errorR_p = countR - countRef;
      errorL_d = errorL_p - old_errorL;            // Difference between the current count delta and the previous: Derivative error
      errorR_d = errorR_p - old_errorR;
      errorL_i = errorL_i + errorL_p + errorL_d;   // Sum of the integral, proportional and derivative errors: Integral error
      errorR_i = errorR_i + errorR_p + errorR_d;      
      old_errorL = errorL_p;                       // Need to store last error to calculate new derivative error on next iteration.
      old_errorR = errorR_p;
    
      forwardSpeedL = ceil((SpeedLadjust - Kp*errorL_p - Kd*errorL_d - Ki*errorL_i));  // Rounds up to the integer speed of the error-adjusted 
      forwardSpeedR = ceil((SpeedRadjust - Kp*errorR_p - Kd*errorR_d - Ki*errorR_i));  // value.  
      forwardSpeedL = constrain(forwardSpeedL,0,255);    // Keep the speeds valid for PWM use.
      forwardSpeedR = constrain(forwardSpeedR,0,255);    
      
      countL=countL-old_countL;      // Reset the count now, since we have finished calculating with it.
      countR=countR-old_countR;      // We'll need this for the next iteration.

      if(DEBUG){
        Serial.print("ErrorL_P: ");
        Serial.print(errorL_p);
        Serial.print("  ErrorR_P: ");
        Serial.println(errorR_p);
        Serial.print("ErrorL_D: ");
        Serial.print(errorL_d);
        Serial.print("  ErrorR_D: ");
        Serial.println(errorR_d);
        Serial.print("ErrorL_I: ");
        Serial.print(errorL_i);
        Serial.print("  ErrorR_I: ");
        Serial.println(errorR_i);
        Serial.print("Left Speed: ");
        Serial.print(forwardSpeedL);
        Serial.print("  Right Speed: ");
        Serial.println(forwardSpeedR);
        Serial.print("Left Count: ");
        Serial.print(countL);
        Serial.print("  Right Count: ");
        Serial.println(countR);
        Serial.println();
      }
    }
}



